package cz.cvut.fel.tar.archivator;

import cz.cvut.fel.tar.archivator.format.processor.*;
import cz.cvut.fel.tar.archivator.customcontrols.*;
import cz.cvut.fel.tar.archivator.processors.*;
import cz.cvut.fel.tar.archivator.transfer.*;
import cz.cvut.fel.tar.archivator.transfer.bluetooth.*;
import cz.cvut.fel.tar.archivator.transfer.filesystem.*;

import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.*;

/**
 * Main application midlet.
 * @author Adam Netocny
 */
public class TarArchivatorMidlet
        extends MIDlet
{

    /**
     * The main application title.
     */
    public static final String APP_DISPLAY_NAME = "TAR Archivator";

    /**
     * Indicates that the midlet is paused.
     */
    private boolean midletPaused = false;

    /**
     * Last screen shown before current.
     */
    private javax.microedition.lcdui.Displayable oldScreen = null;

    /**
     * File browser
     */
    private FileBrowser browser;

    /**
     * Lock for me variable.
     */
    public static final Object lockMe = new Object();

    /**
     * Me. This MIDlet.
     */
    public static TarArchivatorMidlet me;

    /**
     * The TarArchivatorMidlet constructor.
     */
    public TarArchivatorMidlet()
    {
        synchronized (lockMe)
        {
            me = this;
        }
    }

    /**
     * Returns a display instance.
     * @return the display instance.
     */
    public Display getDisplay()
    {
        return Display.getDisplay(this);
    }

    /**
     * Called when MIDlet is started. Checks whether the MIDlet have been
     * already started and initialize/starts or resumes the MIDlet.
     */
    public void startApp()
    {
        if (midletPaused)
        {
            resumeMIDlet();
        }
        else
        {
            initialize();
            startMIDlet();
        }
        midletPaused = false;
    }

    /**
     * Initilizes the application. It is called only once when the MIDlet
     * is started. The method is called before the <code>startMIDlet</code>
     * method.
     */
    private void initialize()
    {
        new FileSystemProcessor();
        new BluetoothTrasferProcessor();
    }

    /**
     * Performs an action assigned to the Mobile Device - MIDlet Started point.
     */
    public void startMIDlet()
    {
        switchDisplayable(getFileBrowser().getScreen());
    }

    /**
     * Called when MIDlet is paused.
     */
    public void pauseApp()
    {
        midletPaused = true;
    }

    /**
     * Performs an action assigned to the Mobile Device - MIDlet Resumed point.
     */
    public void resumeMIDlet()
    {
    }

    /**
     * Called to signal the MIDlet to terminate.
     * @param unconditional if true, then the MIDlet has to be unconditionally
     * terminated and all resources has to be released.
     */
    public void destroyApp(boolean unconditional)
    {
        Enumeration e = FileSystemRegistry.listRoots();
        String root = "";
        while (e.hasMoreElements())
        {
            root = (String) e.nextElement();
        }

        FileConnection conn = null;
        try
        {
            conn = (FileConnection) Connector.open(
                    "file://localhost/" + root + "buffer.tmp");
            if (conn.exists())
            {
                conn.delete();
            }
            conn.close();
        }
        catch (Throwable ex)
        {
        }
    }

    /**
     * Exits MIDlet.
     */
    public void exitMIDlet()
    {
        switchDisplayable(null, null);
        destroyApp(true);
        notifyDestroyed();
    }

    /**
     * Switches a current displayable in a display.
     * @param nextDisplayable The Displayable to be set.
     */
    public void switchDisplayable(
            javax.microedition.lcdui.Displayable nextDisplayable)
    {
        if (nextDisplayable instanceof Alert)
            switchDisplayable((Alert) nextDisplayable, oldScreen);
        else
            switchDisplayable(null, nextDisplayable);
    }

    /**
     * Switches a current displayable in a display. The <code>display</code> 
     * instance is taken from <code>getDisplay</code> method. This method is
     * used by all actions in the design for switching displayable.
     * @param alert the Alert which is temporarily set to the display;
     * if <code>null</code>, then <code>nextDisplayable</code> is set
     * immediately.
     * @param nextDisplayable the Displayable to be set
     */
    public synchronized void switchDisplayable(Alert alert,
            javax.microedition.lcdui.Displayable nextDisplayable)
    {
        Display display = getDisplay();

        oldScreen = display.getCurrent();
        if (alert == null)
        {
            display.setCurrent(nextDisplayable);
        }
        else
        {
            display.setCurrent(alert, nextDisplayable);
        }
    }

    /**
     * Switches to the previous displayable.
     */
    public void screenBack()
    {
        switchDisplayable(oldScreen);
    }

    /**
     * Sets display to the file browser.
     */
    public synchronized void showFileBrowser()
    {
        switchDisplayable(getFileBrowser().getScreen());
    }

    /**
     * Packs files to a tar archive. Will switch to a <tt>PorgressScreen</tt>.
     * @param files Files to be packed.
     */
    public void tarFiles(String[] files)
    {
        if (files == null || files.length == 0)
        {
            switchDisplayable(new AlertScreen("No files were selected!",
                    AlertType.WARNING));
            return;
        }

        Processor p = new TarProcessor(files);
        ProgressScreen sc = new ProgressScreen(
                "Adding files into archive...", p);
        switchDisplayable(sc.getScreen());
    }

    /**
     * Untar file into a destination folder.
     * @param file Tar file to be untared.
     * @param dest Destination folder.
     */
    public void untar(String file, String dest)
    {
        if (file == null || dest == null)
        {
            switchDisplayable(new AlertScreen("No files were selected!",
                    AlertType.WARNING));
            return;
        }

        Processor p = new UnTarProcessor(file, dest);
        ProgressScreen sc = new ProgressScreen(
                "Loading files from archive...", p);
        switchDisplayable(sc.getScreen());
    }

    /**
     * Runs a specific transfer.
     * @param trans Transfer to be started.
     */
    public void runTransfer(Transferable trans)
    {
        ProgressScreen sc = new ProgressScreen("Transfering data through "
                                               + trans.getName() + " chanel....",
                trans);
        switchDisplayable(sc.getScreen());
    }

    /**
     * Gets default file browser.
     * @return File browser screen.
     */
    private FileBrowser getFileBrowser()
    {
        return getFileBrowser(true);
    }

    /**
     * Gets default file browser.
     * @param topView If true the view will be redirected to the top dir.
     * @return File browser screen.
     */
    public FileBrowser getFileBrowser(boolean topView)
    {
        if (browser == null)
            browser = new FileBrowser();

        if (topView)
            browser.setTopDirView();

        return browser;
    }

    /**
     * Reloads current displayable.
     */
    public void updateScreen()
    {
        Display display = getDisplay();

        javax.microedition.lcdui.Displayable dsp = display.getCurrent();
        if (dsp instanceof cz.cvut.fel.tar.archivator.customcontrols.Displayable)
        {
            cz.cvut.fel.tar.archivator.customcontrols.Displayable ndsp =
                    (cz.cvut.fel.tar.archivator.customcontrols.Displayable) dsp;

            display.setCurrent(ndsp.getScreen());
        }
    }
}
