package cz.cvut.fel.tar.archivator.customcontrols;

import cz.cvut.fel.tar.archivator.processors.*;
import cz.cvut.fel.tar.archivator.*;

import javax.microedition.lcdui.*;

/**
 * Screen with progress bar and runs a Runnable with progress.
 * @author Adam Netocny
 */
public class ProgressScreen
        extends Form
        implements CommandListener,
                   Displayable
{

    /**
     * Progress bar.
     */
    private final Gauge progressBar;

    /**
     * Label.
     */
    private final StringItem label;

    /**
     * Thread to be monitored.
     */
    private final ProgressRunnable thread;

    /**
     * Thread which is scanning progress.
     */
    private final ProgressChanger changerThread;

    /**
     * Stop command.
     */
    private final Command stop = new Command("Terminate", Command.BACK, 1);

    /**
     * Next command.
     */
    private final Command next = new Command("Next", Command.OK, 2);

    /**
     * Lock for components.
     */
    private final Object lock = new Object();

    /**
     * Constructor. Runs all threads and adds necessary items into form.
     * @param progressLabel Label for the progress bar.
     * @param thread Thread to be monitored.
     */
    public ProgressScreen(String progressLabel, ProgressRunnable thread)
    {
        super(TarArchivatorMidlet.APP_DISPLAY_NAME);

        this.thread = thread;

        this.progressBar = new Gauge(progressLabel, false,
                this.thread.getMaxProgress(), this.thread.getProgress());
        this.label = new StringItem(null, this.thread.getProgressLabel());

        append(this.progressBar);
        append(label);

        addCommand(stop);

        setCommandListener(this);

        this.changerThread = new ProgressChanger();

        new Thread(this.thread).start();
        new Thread(this.changerThread).start();
    }

    /**
     * Command listener implementation.
     * @param c Received command.
     * @param d Currently shown <tt>Displayable</tt>.
     */
    public void commandAction(Command c, javax.microedition.lcdui.Displayable d)
    {
        if (c == this.stop)
        {
            this.thread.terminateAction();
            this.changerThread.terminate();

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(TarArchivatorMidlet.me.
                        getFileBrowser(true).getScreen());
            }
        }
        else if (c == this.next)
        {
            thread.runPostProcess();
        }
    }

    /**
     * Gets the displayable screen.
     * @return Screen object.
     */
    public Screen getScreen()
    {
        return this;
    }

    /**
     * Progress bar updater thread.
     */
    private class ProgressChanger
            implements Runnable
    {

        /**
         * Indicates that the thread should stop.
         */
        private boolean run = true;

        /**
         * Lock for the run variable.
         */
        private final Object lockRun = new Object();

        /**
         * Monitors progress of the process.
         */
        public void run()
        {
            while (true)
            {
                synchronized (lockRun)
                {
                    if (!run)
                        return;
                }

                synchronized (lock)
                {
                    progressBar.setValue(thread.getProgress());
                    label.setText(thread.getProgressLabel());
                }

                if (thread.getProgress() == thread.getMaxProgress())
                {
                    synchronized (lock)
                    {
                        progressBar.setValue(thread.getProgress());
                        label.setText(thread.getProgressLabel());
                    }
                    synchronized (lockRun)
                    {
                        removeCommand(stop);
                        addCommand(next);
                        return;
                    }
                }

                try
                {
                    Thread.sleep(100);
                }
                catch (InterruptedException ex)
                {
                }
            }
        }

        /**
         * Terminates current process.
         */
        public void terminate()
        {
            synchronized (lockRun)
            {
                run = false;
            }
        }
    }
}
