package cz.cvut.fel.tar.archivator.format.processor;

import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;

/**
 * Simple byte buffer implementation.
 * @author Adam Netocny
 */
public class Buffer
{

    /**
     * Vector for storing byte arrays.
     */
    private final Vector buffer = new Vector();

    /**
     * Global lock for flushing threads.
     */
    private final Object flushLock = new Object();

    /**
     * Indicates that this buffer is flushing to the drive.
     */
    private boolean isFlushing = false;

    /**
     * Bytes red from the temp file.
     */
    private long bytes = 0;

    /**
     * FileConnection to the temp file.
     */
    private FileConnection connTmp = null;

    /**
     * Inserts an byte array.
     * @param buff Byte array to be inserted.
     */
    public void insert(byte[] buff)
    {
        buffer.addElement(buff);
        if (Runtime.getRuntime().freeMemory() < 100)
        {
            if (!isFlushing)
                flushToDrive();
        }
    }

    /**
     * Removes all data from this buffer.
     */
    public void clearBuffer()
    {
        buffer.removeAllElements();

        FileConnection conn = getTmpFileConnection();
        if (conn != null && conn.exists())
        {
            try
            {
                conn.delete();
                conn.close();
                connTmp = null;
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
            }
        }
    }

    /**
     * Reads the first byte array.
     * @return Byte array.
     */
    public byte[] read()
    {
        long size = getTmpFileSize();

        if (size != 0)
        {
            try
            {
                FileConnection conn = getTmpFileConnection();

                InputStream in = (InputStream) conn.openInputStream();

                byte[] buf = new byte[255];

                in.skip(bytes);

                int i = in.read(buf);

                bytes += i;

                byte[] b = new byte[i];
                System.arraycopy(buf, 0, b, 0, i);

                in.close();

                return b;
            }
            catch (Throwable ex)
            {
                ex.printStackTrace();
            }
        }

        if (buffer.isEmpty())
            return null;

        byte[] ret = (byte[]) buffer.firstElement();
        buffer.removeElementAt(0);
        return ret;
    }

    /**
     * Gets size of the first array.
     * @return Size of the first array.
     */
    public int getSizeOfFirst()
    {
        long size = getTmpFileSize();

        if (size != 0)
            return (int) size;

        if (buffer.isEmpty())
            return 0;

        byte[] ret = (byte[]) buffer.firstElement();
        return ret.length;
    }

    /**
     * Gets total size of stored data.
     * @return Total size of stored data.
     */
    public long getTotalSize()
    {
        long size = getTmpFileSize();

        if (buffer.isEmpty())
            return size;


        for (int i = 0; i < buffer.size(); i++)
        {
            byte[] ret = (byte[]) buffer.elementAt(i);
            size += ret.length;
        }
        return size;
    }

    /**
     * Gets size of the temp file.
     * @return Size of the temp file.
     */
    private long getTmpFileSize()
    {
        long size = 0;

        FileConnection conn = getTmpFileConnection();

        if (conn != null)
        {
            try
            {
                if (conn.exists() && !conn.isDirectory())
                    size += conn.fileSize();
            }
            catch (IOException ex)
            {
                ex.printStackTrace();
            }
        }

        return size - bytes;
    }

    /**
     * Gets a <tt>FileConnection</tt> to the temp file.
     * @return <tt>FileConnection</tt> to the temp file.
     */
    private FileConnection getTmpFileConnection()
    {
        if (connTmp != null && connTmp.isOpen())
            return connTmp;

        Enumeration e = FileSystemRegistry.listRoots();
        String root = "";
        while (e.hasMoreElements())
        {
            root = (String) e.nextElement();
        }

        FileConnection conn = null;
        try
        {
            conn = (FileConnection) Connector.open(
                    "file://localhost/" + root + "buffer.tmp");
            if (conn.isDirectory())
            {
                conn.delete();
                conn.close();
                conn = (FileConnection) Connector.open(
                        "file://localhost/" + root + "buffer.tmp");
                conn.create();
            }
        }
        catch (IOException ex)
        {
            ex.printStackTrace();
        }

        connTmp = conn;

        return conn;
    }

    /**
     * Writes the whole content of the buffer into temp file.
     */
    private void flushToDrive()
    {
        isFlushing = true;
        Vector x = new Vector();

        for (int i = 0; i < buffer.size(); i++)
        {
            x.addElement(buffer.elementAt(i));
        }
        buffer.removeAllElements();

        Runtime.getRuntime().gc();
        System.gc();

        try
        {
            if (connTmp != null)
            {
                connTmp.close();
                connTmp = null;
            }

            BufferToFileWriter tmp = new BufferToFileWriter(x);
            tmp.start();
            tmp.join();
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }

        isFlushing = false;

        Runtime.getRuntime().gc();
        System.gc();
    }

    /**
     * Sample thread for writing into temp file.
     */
    private class BufferToFileWriter
            extends Thread
    {

        /**
         * Content of the buffer.
         */
        private final Vector buffer;

        /**
         * Constructor.
         * @param buffer Buffer content.
         */
        public BufferToFileWriter(Vector buffer)
        {
            this.buffer = buffer;
        }

        /**
         * Writes all byte arrays into temp file.
         */
        public void run()
        {
            if (buffer == null || buffer.isEmpty())
                return;

            synchronized (flushLock)
            {
                Enumeration e = FileSystemRegistry.listRoots();
                String root = "";
                while (e.hasMoreElements())
                {
                    root = (String) e.nextElement();
                }

                try
                {
                    FileConnection conn = (FileConnection) Connector.open(
                            "file://localhost/" + root + "buffer.tmp");

                    if (!conn.exists())
                        conn.create();
                    else if (conn.isDirectory())
                    {
                        conn.delete();
                        conn.close();
                        conn = (FileConnection) Connector.open(
                                "file://localhost/" + root + "buffer.tmp");
                        conn.create();
                    }

                    OutputStream out = conn.openOutputStream(conn.fileSize());

                    for (int i = 0; i < buffer.size(); i++)
                    {
                        out.write((byte[]) buffer.elementAt(i));
                    }

                    out.close();
                    conn.close();

                    buffer.removeAllElements();
                    Runtime.getRuntime().gc();
                    System.gc();
                }
                catch (Throwable ex)
                {
                    ex.printStackTrace();
                }
            }
        }
    }
}
