package cz.cvut.fel.tar.archivator.format.processor;

import java.io.*;

/**
 * Implementation of InputStream. Data are red from a buffer.
 * @author Adam Netocny
 */
public class BufferedInputStream
        extends InputStream
{

    /**
     * Buffer were data is stored.
     */
    private Buffer buffer;

    /**
     * Small buffer.
     */
    private byte[] buf;

    /**
     * Constructor.
     * @param buffer Buffer to be red.
     */
    public BufferedInputStream(Buffer buffer)
    {
        this.buffer = buffer;
    }

    /**
     * Gets the buffer.
     * @return Buffer.
     */
    public Buffer getBuffer()
    {
        return buffer;
    }

    /**
     * Reads one byte from the buffer.
     * @return One byte or -1 if nothing is available.
     * @throws IOException If something goes wrong.
     */
    public int read() throws IOException
    {
        if (available() == 0)
        {
            return -1;
        }
        else
        {
            byte[] bytes = getBytes(1);
            return bytes[0];
        }

    }

    /**
     * Reads bytes into a byte array.
     * @param b Byte array.
     * @return Number of bytes red or -1 if nothing is available.
     * @throws IOException If something goes wrong.
     */
    public int read(byte[] b) throws IOException
    {
        return read(b, 0, b.length);
    }

    /**
     * Reads bytes into a byte array.
     * @param b Byte array.
     * @param off Offset in the array.
     * @param len Number of to be red.
     * @return Number of bytes red or -1 if nothing is available.
     * @throws IOException If something goes wrong.
     */
    public int read(byte[] b, int off, int len) throws IOException
    {
        int size = 0;

        if (available() == 0)
        {
            return -1;
        }

        if (available() >= len)
        {
            size = len;
        }
        else
        {
            size = available();
        }

        byte[] bytes = getBytes(size);
        System.arraycopy(bytes, 0, b, off, size);
        return size;
    }

    /**
     * Reset not supported.
     */
    public synchronized void reset()
    {
    }

    /**
     * Closes this stream.
     * @throws IOException If something goes wrong.
     */
    public void close() throws IOException
    {
        buffer.clearBuffer();
    }

    /**
     * Marking not supported. Always false.
     * @return Always false.
     */
    public boolean markSupported()
    {
        return false;
    }

    /**
     * Skips some bytes in the stream, maximal <tt>n</tt>.
     * @param n Maximal number of bytes to be skipped.
     * @return Number of bytes skipped.
     * @throws IOException If something goes wrong.
     */
    public long skip(long n) throws IOException
    {
        int size = 0;

        if (available() == 0)
        {
            return -1;
        }

        if (available() >= (int) n)
        {
            size = (int) n;
        }
        else
        {
            size = available();
        }

        getBytes(size);

        return size;
    }

    /**
     * Gets number of bytes available for reading.
     * @return Number of bytes available for reading.
     * @throws IOException If something goes wrong.
     */
    public int available() throws IOException
    {
        return buffer.getSizeOfFirst() + ((buf != null) ? buf.length : 0);
    }

    /**
     * Reads a byte array with given length.
     * @param size Length of the array.
     * @return Byte array.
     */
    private byte[] getBytes(int size)
    {
        int off = 0;
        byte[] ret = new byte[size];
        if (buf != null)
        {
            int count = 0;
            count = (buf.length > (size - off)) ? (size - off) : buf.length;
            System.arraycopy(buf, 0, ret, off, count);

            off += count;

            if (size - off == 0)
            {
                byte[] tmp = buf;
                count = tmp.length - count;
                buf = new byte[count];
                System.arraycopy(tmp, tmp.length - count, buf, 0, count);
                return ret;
            }
        }

        while (true)
        {
            byte[] tmp = buffer.read();
            int count = 0;
            if (tmp != null)
            {
                count = (tmp.length > (size - off)) ? (size - off) : tmp.length;
                System.arraycopy(tmp, 0, ret, off, count);

                off += count;

                if (size - off == 0)
                {
                    count = tmp.length - count;
                    buf = new byte[count];
                    System.arraycopy(tmp, tmp.length - count, buf, 0, count);
                    break;
                }
            }
        }

        return ret;
    }
}
