package cz.cvut.fel.tar.archivator.format.processor;

import java.io.*;

/**
 * Implementation of OutputStream. Date is written into buffer.
 * @author Adam Netocny
 */
public class BufferedOutputStream
        extends OutputStream
{

    /**
     * Maximal block size for buffer.
     */
    private static final int MAX_BLOCK_SIZE = 255;

    /**
     * Buffer.
     */
    private final Buffer buffer = new Buffer();

    /**
     * Block of data. Will be stored after flush, close or if it's full.
     */
    private byte[] arr = new byte[MAX_BLOCK_SIZE];

    /**
     * Sample index for arr.
     */
    private int index = 0;

    /**
     * Writes one byte into buffer.
     * @param b Byte to be written.
     * @throws IOException If something goes wrong.
     */
    public void write(int b) throws IOException
    {
        arr[index++] = (byte) b;
        if (index == MAX_BLOCK_SIZE)
        {
            buffer.insert(arr);
            arr = new byte[MAX_BLOCK_SIZE];
            index = 0;
        }
    }

    /**
     * Writes byte array into buffer. Date from this array are red with offset
     * <tt>off</tt> and with length <tt>len</tt>.
     * @param b Byte array.
     * @param off Offset.
     * @param len Length.
     * @throws IOException If something goes wrong.
     */
    public void write(byte[] b, int off, int len) throws IOException
    {
        if ((off | len | (b.length - (len + off)) | (off + len)) < 0)
            throw new IndexOutOfBoundsException();

        for (int i = 0; i < len; i++)
        {
            write(b[off + i]);
        }
    }

    /**
     * Writes byte array into buffer.
     * @param b Byte array.
     * @throws IOException If something goes wrong.
     */
    public void write(byte[] b) throws IOException
    {
        write(b, 0, b.length);
    }

    /**
     * Sample flush implementation.
     * @throws IOException If something goes wrong.
     */
    public void flush() throws IOException
    {
        byte[] tmp = new byte[index];
        System.arraycopy(arr, 0, tmp, 0, index);
        buffer.insert(tmp);
        arr = new byte[MAX_BLOCK_SIZE];
        index = 0;
    }

    /**
     * Closes the stream.
     * @throws IOException If something goes wrong.
     */
    public void close() throws IOException
    {
        flush();
    }

    /**
     * Gets the buffer with stored data.
     * @return Buffer.
     */
    public Buffer getBuffer()
    {
        return buffer;
    }
}
