package cz.cvut.fel.tar.archivator.format.processor;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.customcontrols.*;
import cz.cvut.fel.tar.archivator.format.*;
import cz.cvut.fel.tar.archivator.processors.*;
import cz.cvut.fel.tar.archivator.transfer.calculator.TransferSpeedCalculator;

import java.io.*;
import java.util.*;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;

/**
 * Archivates files into tar format.
 * @author Adam Netocny
 */
public class TarProcessor
        extends Processor
{

    /**
     * Termination variable.
     */
    private boolean run = true;

    /**
     * Lock for run variable.
     */
    private final Object lockRun = new Object();

    /**
     * Files to be processed.
     */
    private String[] files;

    /**
     * Result of this operation.
     */
    private BufferedOutputStream result;

    /**
     * Number of bytes processed.
     */
    private int processed = 0;

    /**
     * Label.
     */
    private String label = "b of 0b";

    /**
     * Output of calculator.
     */
    private String calculatorOutput = "";

    /**
     * Lock for calculator output.
     */
    private final Object calculatorOutputLock = new Object();

    /**
     * Constructor. Setups all necessary variables.
     * @param files Files to be processed.
     */
    public TarProcessor(String[] files)
    {
        this.files = files;
    }

    /**
     * Terminates this thread as soon as possible.
     */
    public void terminateAction()
    {
        synchronized (lockRun)
        {
            if (result != null)
            {
                result.getBuffer().clearBuffer();
            }
            run = false;
        }
    }

    /**
     * Postprocess action.
     */
    public void runPostProcess()
    {
        try
        {
            Buffer buf = result.getBuffer();
            result = null;
            System.gc();

            synchronized (TarArchivatorMidlet.lockMe)
            {

                TarArchivatorMidlet.me.switchDisplayable(
                        new TransfersSelector(
                        buf).getScreen());
            }
        }
        catch (Throwable ex)
        {
            final Throwable exx = ex;
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(
                        new AlertScreen(exx.getMessage(),
                        AlertType.ERROR));
            }
        }
    }

    /**
     * Adds files into tar.
     */
    public void run()
    {
        try
        {
            if (files == null)
            {
                synchronized (lock)
                {
                    progress = getMaxProgress();
                }
                return;
            }

            if (files.length == 0)
            {
                synchronized (lock)
                {
                    progress = getMaxProgress();
                }
                return;
            }

            long size = calculateSize();

            TransferSpeedCalculator transferSpeedCalculator = new TransferSpeedCalculator(
                    size);

            label = "b of " + size + "b";
            synchronized (lockRun)
            {
                if (!run)
                {
                    return;
                }
            }


            TarOutputStream out = new TarOutputStream(getTmpOutputStream());

            for (int i = 0; i < files.length; i++)
            {
                synchronized (lockRun)
                {
                    if (!run)
                    {
                        return;
                    }
                }

                FileConnection conn = (FileConnection) Connector.open(
                        "file://localhost/" + files[i]);

                if (conn.isDirectory())
                {
                    out.putNextEntry(new TarEntry(conn, files[i]));
                    appendFolder(conn, files[i]);
                    conn.close();
                    continue;
                }

                InputStream in = conn.openInputStream();

                out.putNextEntry(new TarEntry(conn, files[i]));

                byte[] buffer = new byte[100];
                int c;
                while ((c = in.read(buffer)) != -1)
                {
                    out.write(buffer, 0, c);

                    synchronized (calculatorOutputLock)
                    {
                        calculatorOutput = transferSpeedCalculator.
                                calculateRemainingTime(c);
                    }

                    synchronized (lock)
                    {
                        processed += c;
                        progress = new Double(
                                ((double) processed) / ((double) size) * 100).
                                intValue();
                    }
                    synchronized (lockRun)
                    {
                        if (!run)
                        {
                            return;
                        }
                    }
                }

                in.close();
                conn.close();
            }

            out.flush();
            out.close();

            synchronized (lock)
            {
                progress = getMaxProgress();
            }
        }
        catch (Throwable ex)
        {
            final Throwable exx = ex;

            synchronized (lock)
            {
                progress = getMaxProgress();
            }

            if (result != null)
            {
                result.getBuffer().clearBuffer();
            }

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(
                        new AlertScreen(exx.getMessage(),
                        AlertType.ERROR));
            }

            return;
        }
    }

    /**
     * Calculates size of all files.
     * @return Size of all files.
     * @throws IOException If something goes wrong.
     */
    private long calculateSize() throws IOException
    {
        long size = 0;

        for (int i = 0; i < files.length; i++)
        {
            FileConnection conn = (FileConnection) Connector.open(
                    "file://localhost/" + files[i]);
            if (conn.isDirectory())
            {
                size += conn.directorySize(true);
            }
            else
            {
                size += conn.fileSize();
            }

            if (conn != null)
            {
                conn.close();
            }
        }

        return size;
    }

    /**
     * Creates a new <tt>OutputStream</tt> for this operation. This implementation is
     * using a <tt>BufferedOutputStream</tt>.
     * @return OutputStream.
     *
     * @see BufferedOutputStream
     */
    private OutputStream getTmpOutputStream()
    {
        result = new BufferedOutputStream();
        return result;
    }

    /**
     * Append folder content to the list of files.
     * @param conn <tt>FileConnection</tt> to the folder.
     * @param string Path of the folder.
     */
    private void appendFolder(FileConnection conn, String string)
    {
        String[] f = null;
        try
        {
            Enumeration e = conn.list();
            Vector x = new Vector();
            while (e.hasMoreElements())
            {
                x.addElement(string + e.nextElement());
            }

            f = new String[x.size()];
            for (int i = 0; i < x.size(); i++)
            {
                f[i] = (String) x.elementAt(i);
            }
        }
        catch (IOException ex)
        {
        }

        if (f != null)
        {
            String[] f2 = files;

            files = new String[f.length + f2.length];

            System.arraycopy(f2, 0, files, 0, f2.length);
            System.arraycopy(f, 0, files, f2.length, f.length);
        }
    }

    /**
     * Gets string label to be shown with the progress bar.
     * @return String label.
     */
    public String getProgressLabel()
    {
        synchronized (calculatorOutputLock)
        {
            return super.getProgressLabel() + " Total: " + processed + label + "\n" + calculatorOutput;
        }
    }
}
