package cz.cvut.fel.tar.archivator.format.processor;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.customcontrols.*;
import cz.cvut.fel.tar.archivator.format.*;
import cz.cvut.fel.tar.archivator.processors.*;
import cz.cvut.fel.tar.archivator.transfer.calculator.*;
import java.io.*;
import java.util.Enumeration;
import javax.microedition.io.*;
import javax.microedition.io.file.*;
import javax.microedition.lcdui.*;

/**
 * Untars files into a destination folder.
 * @author Adam Netocny
 */
public class UnTarProcessor
        extends Processor
{

    /**
     * Termination variable.
     */
    private boolean run = true;

    /**
     * Lock for run variable.
     */
    private final Object lockRun = new Object();

    /**
     * File to be processed.
     */
    private String file;

    /**
     * Number of bytes processed.
     */
    private int processed = 0;

    /**
     * Output of calculator.
     */
    private String calculatorOutput = "";

    /**
     * Lock for calculator output.
     */
    private final Object calculatorOutputLock = new Object();

    /**
     * Destination folder.
     */
    private String dest;

    /**
     * Constructor. Setups all necessary variables.
     * @param file File to be processed.
     * @param dest
     */
    public UnTarProcessor(String file, String dest)
    {
        this.file = file;
        this.dest = dest;
    }

    /**
     * Terminates this thread as soon as possible.
     */
    public void terminateAction()
    {
        synchronized (lockRun)
        {
            run = false;
        }
    }

    /**
     * Postprocess action.
     */
    public void runPostProcess()
    {
        try
        {
            System.gc();

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.screenBack();
            }
            new Thread()
            {

                public void run()
                {
                    synchronized (TarArchivatorMidlet.lockMe)
                    {
                        TarArchivatorMidlet.me.switchDisplayable(new AlertScreen(
                                "The archive was unpacked into: " + dest,
                                AlertType.INFO));
                    }
                }
            }.start();
        }
        catch (Throwable ex)
        {
            final Throwable exx = ex;
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(
                        new AlertScreen(exx.getMessage(),
                        AlertType.ERROR));
            }
        }
    }

    /**
     * Adds files into tar.
     */
    public void run()
    {
        try
        {
            if (file == null)
            {
                synchronized (lock)
                {
                    progress = getMaxProgress();
                }
                return;
            }

            FileConnection conn = (FileConnection) Connector.open(
                    "file://localhost/" + file);
            long size = conn.fileSize();
            conn.close();

            TransferSpeedCalculator transferSpeedCalculator = new TransferSpeedCalculator(
                    size);

            synchronized (lockRun)
            {
                if (!run)
                {
                    rollBack();
                    return;
                }
            }

            TarInputStream in = new TarInputStream(Connector.openInputStream(
                    "file://localhost/" + file));
            TarEntry entry;
            while ((entry = in.getNextEntry()) != null)
            {
                synchronized (lockRun)
                {
                    if (!run)
                    {
                        rollBack();
                        return;
                    }
                }

                int count;
                byte data[] = new byte[2048];

                String entryName = entry.getName().replace(':', '_');
                prepareFolders(entryName);
                if (entry.isDirectory())
                {
                    continue;
                }

                createFileIfNotExists(
                        "file://localhost/" + dest + "/" + entryName);
                OutputStream out = (OutputStream) Connector.openOutputStream(
                        "file://localhost/" + dest + "/" + entryName);

                while ((count = in.read(data)) != -1)
                {
                    synchronized (calculatorOutputLock)
                    {
                        calculatorOutput = transferSpeedCalculator.
                                calculateRemainingTime(count);
                    }

                    synchronized (lock)
                    {
                        processed += count;
                        progress = new Double(
                                ((double) processed) / ((double) size) * 100).
                                intValue();
                    }
                    out.write(data, 0, count);
                }

                out.flush();
                out.close();
            }

            in.close();

            synchronized (calculatorOutputLock)
            {
                calculatorOutput = transferSpeedCalculator.
                        calculateRemainingTime(size - processed);
            }

            synchronized (lock)
            {
                progress = getMaxProgress();
            }
        }
        catch (Throwable ex)
        {
            final Throwable exx = ex;

            synchronized (lock)
            {
                progress = getMaxProgress();
            }

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(
                        new AlertScreen(exx.getMessage(),
                        AlertType.ERROR));
            }

            return;
        }
    }

    /**
     * Gets string label to be shown with the progress bar.
     * @return String label.
     */
    public String getProgressLabel()
    {
        synchronized (calculatorOutputLock)
        {
            return super.getProgressLabel() + "\n" + calculatorOutput;
        }
    }

    /**
     * Creates all directories in path.
     * @param name Path to a file.
     * @throws IOException ---------
     */
    private void prepareFolders(String name) throws IOException
    {
        FileConnection conn = (FileConnection) Connector.open(
                "file://localhost/" + dest);
        if (conn.exists() && !conn.isDirectory())
        {
            conn.delete();
        }
        if (!conn.exists())
        {
            conn.mkdir();
        }
        conn.close();

        String folder = "";
        while (name.indexOf('/') != -1)
        {
            folder = folder + name.substring(0, name.indexOf('/') + 1);
            name = name.substring(name.indexOf('/') + 1);
            conn = (FileConnection) Connector.open(
                    "file://localhost/" + dest + "/" + folder);
            if (conn.exists() && !conn.isDirectory())
            {
                conn.delete();
            }
            if (!conn.exists())
            {
                conn.mkdir();
            }
            conn.close();
        }
    }

    /**
     * Creates an file if it doesn't exist.
     * @param string File path.
     * @throws IOException -----------
     */
    private void createFileIfNotExists(String string) throws IOException
    {
        FileConnection conn = (FileConnection) Connector.open(string,
                Connector.READ_WRITE);
        if (conn.exists() && conn.isDirectory())
        {
            conn.delete();
        }
        if (!conn.exists())
        {
            conn.create();
        }
        conn.close();
    }

    /**
     * Removes created files and folders.
     * @throws IOException ------------
     */
    private void rollBack() throws IOException
    {
        FileConnection conn = (FileConnection) Connector.open(
                "file://localhost/" + dest + "/");
        if (conn.exists() && conn.isDirectory())
        {
            conn.close();
            deleteDir(dest);
            return;
        }
        conn.close();
    }

    /**
     * Removes a folder.
     * @param dest Path to the folder.
     * @throws IOException ------------------
     */
    private void deleteDir(String dest) throws IOException
    {
        FileConnection conn = (FileConnection) Connector.open(
                "file://localhost/" + dest + "/");
        Enumeration list = conn.list();

        while (list.hasMoreElements())
        {
            String fileName = (String) list.nextElement();
            if (fileName.charAt(fileName.length() - 1) == '/')
            {
                deleteDir(dest + "/" + fileName);
            }
            else
            {
                FileConnection connFile = (FileConnection) Connector.open(
                        "file://localhost/" + dest + "/" + fileName);
                connFile.delete();
                connFile.close();
            }
        }
        conn.delete();
        conn.close();
    }
}
