package cz.cvut.fel.tar.archivator.transfer.bluetooth;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.customcontrols.*;

import java.io.*;
import javax.bluetooth.*;
import javax.microedition.lcdui.*;

/**
 * Bluetooth selector for selecting of of available devices.
 * @author Adam Netocny
 */
class BluetoothSelector
        extends List
        implements cz.cvut.fel.tar.archivator.customcontrols.Displayable,
                   CommandListener
{

    /**
     * Bluetooth logo.
     */
    private Image bt_image = null;

    /**
     * Select command.
     */
    private final Command select = new Command("Select", Command.ITEM, 1);

    /**
     * Refresh command.
     */
    private final Command refresh = new Command("Refresh", Command.ITEM, 2);

    /**
     * Exit command.
     */
    private final Command exit = new Command("Exit", Command.EXIT, 3);

    /**
     * Service url to be returned to the <tt>BluetoothTrasferProcessor</tt>.
     *
     * @see BluetoothTrasferProcessor
     */
    private String service = null;

    /**
     * Lock for discovery variable.
     */
    private final Object lockDiscovery = new Object();

    /**
     * Device discovery service.
     */
    private BluethootDeviceDiscovery discovery = null;

    /**
     * Constructor. Initializes default values and starts searching process.
     */
    public BluetoothSelector()
    {
        super(TarArchivatorMidlet.APP_DISPLAY_NAME
              + " - Available devices", IMPLICIT);

        try
        {
            bt_image = Image.createImage("/resources/bluetooth.png");
        }
        catch (IOException ex)
        {
        }

        new Thread()
        {

            public void run()
            {
                refresh();
            }
        }.start();

        addCommand(exit);
        setCommandListener(this);
    }

    /**
     * Appends string to the list with the default Bluetooth image.
     * @param stringPart String to be appended.
     * @return The assigned index of the element.
     */
    public synchronized int append(String stringPart)
    {
        return super.append(stringPart, bt_image);
    }

    /**
     * Is called when the search process ends.
     */
    public void searchComplete()
    {
        if (size() == 0)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(new AlertScreen(
                        "No devices available.", AlertType.INFO).getScreen());
            }
        }
        else
        {
            setSelectCommand(select);
        }

        setTitle(TarArchivatorMidlet.APP_DISPLAY_NAME + " - "
                 + "Available devices");
        addCommand(refresh);
    }

    /**
     * Gets the displayable screen.
     * @return Screen object.
     */
    public Screen getScreen()
    {
        return this;
    }

    /**
     * Gets the url of the service.
     * @return Url of the service.
     */
    public String getService()
    {
        return service;
    }

    /**
     * Sets the url of the service.
     * @param service Url of the service.
     */
    synchronized void setService(String service)
    {
        this.service = service;
    }

    /**
     *
     */
    public void refresh()
    {
        setTitle(TarArchivatorMidlet.APP_DISPLAY_NAME + " - Searching....");
        removeCommand(refresh);
        deleteAll();
        setSelectCommand(SELECT_COMMAND);

        synchronized (lockDiscovery)
        {
            discovery = new BluethootDeviceDiscovery(this);
        }
    }

    /**
     * Command listener implementation.
     * @param c Received command.
     * @param d Currently shown <tt>Displayable</tt>.
     */
    public void commandAction(Command c, javax.microedition.lcdui.Displayable d)
    {
        if (c == refresh)
        {
            refresh();
        }
        else if (c == select)
        {
            int index = getSelectedIndex();

            if (index != -1)
            {
                RemoteDevice dev;
                synchronized (lockDiscovery)
                {
                    dev = (RemoteDevice) discovery.getRemoteDevices().elementAt(
                            index);
                }
                new BluetoothServiceDiscovery(this, dev).start();
            }
        }
        else if (c == exit)
        {
            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.exitMIDlet();
            }
        }
    }
}
