package cz.cvut.fel.tar.archivator.transfer.bluetooth;

import cz.cvut.fel.tar.archivator.*;
import cz.cvut.fel.tar.archivator.customcontrols.*;
import cz.cvut.fel.tar.archivator.format.processor.*;
import cz.cvut.fel.tar.archivator.transfer.*;
import cz.cvut.fel.tar.archivator.transfer.calculator.TransferSpeedCalculator;

import java.io.*;
import javax.microedition.io.*;
import javax.microedition.lcdui.*;
import javax.obex.*;

/**
 * This class implements the bluetooth transfer processor.
 * @author Adam Netocny
 */
public class BluetoothTrasferProcessor
        extends TransferProcessor
{

    /**
     * Termination variable.
     */
    private boolean run = true;

    /**
     * Lock for run variable.
     */
    private final Object lockRun = new Object();

    /**
     * Number of bytes written to the file.
     */
    private long written = 0;

    /**
     * Output of calculator.
     */
    private String calculatorOutput = "";

    /**
     * Lock for calculator output.
     */
    private final Object calculatorOutputLock = new Object();

    /**
     * Terminates this action as soon as possible.
     */
    public void terminateAction()
    {
        synchronized (lockRun)
        {
            run = false;
        }
    }

    /**
     * Opens selector screen and after selection is made, transfers data through
     * OBEX chanel.
     */
    public void run()
    {
        try
        {
            BluetoothSelector selector = new BluetoothSelector();

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(selector.getScreen());
            }

            while (selector.getService() == null)
            {
                Thread.sleep(100);
            }

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.screenBack();
            }


            if (selector == null || selector.getScreen() == null)
            {
                synchronized (TarArchivatorMidlet.lockMe)
                {
                    TarArchivatorMidlet.me.switchDisplayable(new AlertScreen(
                            "Unknown error.", AlertType.ERROR).getScreen());
                }

                TransfersRegister.unregisterTransfer(
                        BluetoothTrasferProcessor.class);

                new BluetoothTrasferProcessor();

                in.clearBuffer();
                return;
            }

            String url = selector.getService();

            long size = in.getTotalSize();

            TransferSpeedCalculator speedCalculator = new TransferSpeedCalculator(
                    size);

            BufferedInputStream bufin = new BufferedInputStream(in);

            in = null;

            Connection connection = Connector.open(url);

            // now, let's create a session and a headerset objects
            ClientSession cs = (ClientSession) connection;
            HeaderSet hs = cs.createHeaderSet();

            // now let's send the connect header
            cs.connect(hs);

            hs.setHeader(HeaderSet.NAME, "tar_archivator.tar");
            hs.setHeader(HeaderSet.TYPE, "application/x-tar");
            hs.setHeader(HeaderSet.LENGTH, new Long(size));

            Operation putOperation = cs.put(hs);

            OutputStream out = putOperation.openOutputStream();

            byte[] b = new byte[255];
            int c = 0;
            while ((c = bufin.read(b)) != -1)
            {
                out.write(b, 0, c);

                synchronized (calculatorOutputLock)
                {
                    calculatorOutput = speedCalculator.calculateRemainingTime(c);
                }

                synchronized (lock)
                {
                    written += c;
                    progress = new Double(
                            ((double) written) / ((double) size) * 100).intValue();
                }
                synchronized (lockRun)
                {
                    if (!run)
                    {
                        ((BufferedInputStream) bufin).getBuffer().clearBuffer();

                        out.close();
                        putOperation.close();
                        cs.disconnect(null);
                        connection.close();
                        bufin.close();
                        in.clearBuffer();

                        Thread.sleep(1000);

                        TransfersRegister.unregisterTransfer(
                                BluetoothTrasferProcessor.class);

                        new BluetoothTrasferProcessor();
                        return;
                    }
                }
            }

            out.close();
            putOperation.close();
            cs.disconnect(null);
            connection.close();
            bufin.close();

            Thread.sleep(1000);

            TransfersRegister.unregisterTransfer(
                    BluetoothTrasferProcessor.class);

            new BluetoothTrasferProcessor();
        }
        catch (Throwable ex)
        {
            TransfersRegister.unregisterTransfer(
                    BluetoothTrasferProcessor.class);

            new BluetoothTrasferProcessor();
            in.clearBuffer();

            synchronized (TarArchivatorMidlet.lockMe)
            {
                TarArchivatorMidlet.me.switchDisplayable(new AlertScreen(
                        ex.getMessage(), AlertType.ERROR).getScreen());
            }
        }
    }

    /**
     * Gets the name of this transfer.
     * @return Name of this transfer.
     */
    public String getName()
    {
        return "Bluetooth/OBEX";
    }

    /**
     * Gets the icon for this transfer type.
     * @return Icon for this transfer type.
     */
    public Image getIcon()
    {
        Image folder = null;
        try
        {
            folder = Image.createImage("/resources/bluetooth.png");
        }
        catch (IOException ex)
        {
        }

        return folder;
    }

    /**
     * Gets label to be shown under progress bar.
     * @return String label.
     */
    public String getProgressLabel()
    {
        synchronized (calculatorOutputLock)
        {
            return super.getProgressLabel() + "\n" + calculatorOutput;
        }
    }
}
